@@ -46,11 +46,11 @@ def setUpModule():  # noqa: N802
     if shutil.which('coverage'):
         coverage_cli_args = shlex.split('coverage run --rcfile {}'.format(os.path.join(base_path, '.coveragerc')))
     else:
-        logging.warning("Couldn't find `coverage` executable, not running coverage tests!")
+        logging.warning("Couldn't find `coverage` executable, not running server with coverage instrumentation!")
         for _ in range(3):
             time.sleep(1)
             print('.')
-    server_handle = subprocess.Popen(coverage_cli_args + [os.path.join(base_path, 'servlet.py')] + cli_args)  # TODO: print only on error?
+    server_handle = subprocess.Popen(coverage_cli_args + [os.path.join(base_path, 'apy.py')] + cli_args)  # TODO: print only on error?
 
     started = False
     waited = 0
@@ -80,7 +80,7 @@ class BaseTestCase(AsyncHTTPTestCase):
     def get_http_port(self):
         return PORT
 
-    def fetch(self, path, params={}, **kwargs):
+    def fetch(self, path, params={}, **kwargs):  # type: ignore[override]
         if params:
             method = kwargs.get('method', 'GET')
             if method == 'GET':
@@ -414,12 +414,11 @@ class TestListLanguageNamesHandler(BaseTestCase):
         response = self.fetch_json('/listLanguageNames')
         self.assertEqual(response['en'], 'English')
 
-    @unittest.skip('Failing for unknown reasons')
     def test_accept_languages_header_lang_names_list(self):
         response = self.fetch_json('/listLanguageNames', headers={
             'Accept-Language': 'fr-CH, fr;q=0.9, en;q=0.8, de;q=0.7, *;q=0.5',
         })
-        self.assertEqual(response['en'], 'Anglais')
+        self.assertEqual(response['en'], 'anglais')
 
 
 class TestIdentifyLangHandler(BaseTestCase):
@@ -452,5 +451,25 @@ class TestCoverageHandler(BaseTestCase):
         })
 
 
+class TestPipeDebugHandler(BaseTestCase):
+    def test_pipe_debug(self):
+        response = self.fetch_json('/pipedebug', {'q': 'house', 'langpair': 'eng|spa'})
+        self.assertEqual(response['responseStatus'], 200)
+
+        pipeline = response['responseData']['pipeline']
+        output = response['responseData']['output']
+        self.assertEqual(len(pipeline) + 1, len(output))
+        self.assertGreater(len(output), 10)
+
+    def test_invalid_mode(self):
+        response = self.fetch_json('/pipedebug', {'q': 'ignored', 'langpair': 'eng-spa'}, expect_success=False)
+        self.assertDictEqual(response, {
+            'status': 'error',
+            'code': 400,
+            'message': 'Bad Request',
+            'explanation': 'That pair is invalid, use e.g. eng|spa',
+        })
+
+
 if __name__ == '__main__':
     unittest.main()
